Option Explicit

Dim g_argsNamed
Dim g_tsvars

Dim g_manufacturerLookup
Dim g_serialLookup

Set g_argsNamed = WScript.Arguments.Named
Set g_manufacturerLookup = 	CreateObject ("Scripting.Dictionary")
Set g_serialLookup = 		CreateObject ("Scripting.Dictionary")

' Add to or modify these
'  The first string is serached for in the Manufacturer attribute of the Win32_BIOS and the second string is used for the code if it is found
g_manufacturerLookup.Add "Gateway", 			"GW"
g_manufacturerLookup.Add "HP", 					"HP"
g_manufacturerLookup.Add "Lenovo",				"LE"
g_manufacturerLookup.Add "Dell", 				"DE"

' Add to or modify these
'  Based on the manufacturer code specified by the first string, the class and attribute specified in the second string are queried for the serial number
g_serialLookup.Add "GW", "Win32_Bios,SerialNumber"
g_serialLookup.Add "HP", "Win32_Bios,SerialNumber"
g_serialLookup.Add "LE", "Win32_Bios,SerialNumber"
g_serialLookup.Add "DE", "Win32_Bios,SerialNumber"

' The following object will not exist outside of a task sequence
On Error Resume Next
Set g_tsvars = CreateObject ("Microsoft.SMS.TSEnvironment")
On Error GoTo 0

Main

Sub Main

	Dim modelCode, serialNumber, chassisCode, computerName
	
	modelCode = GetModelCode
	
	serialNumber = GetSerial (modelCode)
	
	chassisCode = GetChassisCode
	
	computerName = modelCode & "-" & chassisCode & "-" & serialNumber
	
	If Not IsObject(g_tsvars) Then
		WScript.Echo computerName
	Else
		g_tsvars("OSDComputerName") = computerName
	End If
	

End Sub

Function GetWMIAttribute (ByVal attr, ByVal wmiClass)
	Dim objList, item, returnValue
	
	Set objList = GetObject("winmgmts:").InstancesOf (wmiClass)
	
	On Error Resume Next

	For Each item In objList

		returnValue = item.Properties_(attr)

		If Err <> 0 Then
			returnValue = "Error: " & Err.Description
		End If

	Next
		
	On Error GoTo 0
	
	GetWMIAttribute = Trim(returnValue)
	
End Function

Function GetModelCode

	Dim manufacturer
	Dim value
	
	manufacturer = GetWMIAttribute ("Manufacturer", "Win32_ComputerSystem")
	
	For Each value In g_manufacturerLookup.Keys
		If InStr (manufacturer, value) > 0 Then
			GetModelCode = g_manufacturerLookup.Item(value)
			Exit Function
		End If
	Next

End Function

Function GetChassisCode
	Dim results, instance
	
	Set results = GetObject("winmgmts:").InstancesOf("Win32_SystemEnclosure")
	
	For each instance in results
	
		If instance.ChassisTypes(0) = 12 or instance.ChassisTypes(0) = 21 Then
		' Ignore docking stations
		Else
		
			Select Case instance.ChassisTypes(0)
				Case "8", "9", "10", "11", "12", "14", "18", "21"
					GetChassisCode = "L"
				Case "3", "4", "5", "6", "7", "15", "16"
					GetChassisCode = "D"
				Case "23"
					GetChassisCode = "S"
				Case Else
				' Do nothing
			End Select
		
		End If
	
	Next
End Function

Function GetSerial (ByVal model)

	Dim lookupString, wmiValues

	If g_serialLookup.Exists(model) Then
		lookupString = g_serialLookup.Item(model)
		
		wmiValues = Split(lookupString, ",")
		
		GetSerial = GetWMIAttribute(wmiValues(1), wmiValues(0))
		
	End If
End Function
